﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Infragistics.Win;
using Infragistics.Win.UltraWinEditors;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Drawing.Drawing2D;

namespace ControlPropertyDefaultValue
{
    #region CheckBoxGlyphSettings class
    [Description("Exposes properties which facilitate the generation of glyph images for a CheckBox.")]
    public class CheckBoxGlyphSettings : GlyphSettings
    {

        private List<UltraCheckEditor> _checkEditorList = null;
        private GlyphAppearances _indeterminateAppearances = null;

        public CheckBoxGlyphSettings() : base(GlyphType.CheckBox) { }

        protected override void OnPropChange(object sender, EventArgs e)
        {
            GlyphInfoBase glyphInfo = this.ToGlyphInfo();

            if (this._checkEditorList != null)
            {
                for (int i = 0; i < this._checkEditorList.Count; i++)
                {
                    UltraCheckEditor control = this._checkEditorList[i];
                    control.GlyphInfo = glyphInfo;
                    control.UIElement.DirtyChildElements(true);
                }
            }
        }

        public void Register(List<UltraCheckEditor> checkEditorList)
        {
            this._checkEditorList = checkEditorList;
        }

        public override GlyphInfoBase ToGlyphInfo()
        {
            this.DisposeImages();

            CheckBoxImageGlyphInfo glyphInfo = new CheckBoxImageGlyphInfo(
                this.Size,
                this.GetImage(CheckState.Unchecked, VisualState.Disabled),
                this.GetImage(CheckState.Checked, VisualState.Disabled),
                this.GetImage(CheckState.Indeterminate, VisualState.Disabled),
                this.GetImage(CheckState.Unchecked, VisualState.Normal),
                this.GetImage(CheckState.Checked, VisualState.Normal),
                this.GetImage(CheckState.Indeterminate, VisualState.Normal),
                this.GetImage(CheckState.Unchecked, VisualState.MouseOver),
                this.GetImage(CheckState.Checked, VisualState.MouseOver),
                this.GetImage(CheckState.Indeterminate, VisualState.MouseOver),
                this.GetImage(CheckState.Unchecked, VisualState.Pressed),
                this.GetImage(CheckState.Checked, VisualState.Pressed),
                this.GetImage(CheckState.Indeterminate, VisualState.Pressed),
                this.GetImage(CheckState.Unchecked, VisualState.PressedAndOver),
                this.GetImage(CheckState.Checked, VisualState.PressedAndOver),
                this.GetImage(CheckState.Indeterminate, VisualState.PressedAndOver));

            return glyphInfo;
        }

        [Description("Returns a GlyphAppearances object specific to the 'Indeterminate' state.")]
        [Category("Appearance")]
        public GlyphAppearances IndeterminateAppearances
        {
            get
            {
                if (this._indeterminateAppearances == null)
                    this._indeterminateAppearances = new GlyphAppearances(CheckState.Indeterminate, new EventHandler(this.OnPropChange));

                return this._indeterminateAppearances;
            }
        }

        protected override GlyphAppearances GetAppearances(CheckState checkState)
        {
            return checkState == CheckState.Indeterminate ? this.IndeterminateAppearances : base.GetAppearances(checkState);
        }
    }
    #endregion CheckBoxGlyphSettings class

    #region RadioButtonGlyphSettings class
    [Description("Exposes properties which facilitate the generation of glyph images for a RadioButton.")]
    public class RadioButtonGlyphSettings : GlyphSettings
    {
        private List<UltraOptionSet> _optionSetList = null;

        public RadioButtonGlyphSettings() : base(GlyphType.RadioButton) { }

        protected override void OnPropChange(object sender, EventArgs e)
        {
            GlyphInfoBase glyphInfo = this.ToGlyphInfo();

            if (this._optionSetList != null)
            {
                for (int i = 0; i < this._optionSetList.Count; i++)
                {
                    UltraOptionSet control = this._optionSetList[i];
                    control.GlyphInfo = glyphInfo;
                    control.UIElement.DirtyChildElements(true);
                }
            }
        }

        public void Register(List<UltraOptionSet> optionSetList)
        {
            this._optionSetList = optionSetList;
        }

        public override GlyphInfoBase ToGlyphInfo()
        {
            this.DisposeImages();

            RadioButtonImageGlyphInfo glyphInfo = new RadioButtonImageGlyphInfo(
                this.Size,
                this.GetImage(CheckState.Unchecked, VisualState.Disabled),
                this.GetImage(CheckState.Checked, VisualState.Disabled),
                this.GetImage(CheckState.Unchecked, VisualState.Normal),
                this.GetImage(CheckState.Checked, VisualState.Normal),
                this.GetImage(CheckState.Unchecked, VisualState.MouseOver),
                this.GetImage(CheckState.Checked, VisualState.MouseOver),
                this.GetImage(CheckState.Unchecked, VisualState.Pressed),
                this.GetImage(CheckState.Checked, VisualState.Pressed),
                this.GetImage(CheckState.Unchecked, VisualState.PressedAndOver),
                this.GetImage(CheckState.Checked, VisualState.PressedAndOver));

            return glyphInfo;
        }

    }
    #endregion RadioButtonGlyphSettings class

    #region GlyphSettings class
    [Description("Exposes properties which facilitate the generation of glyph images for a CheckBox or RadioButton.")]
    [DefaultProperty("About")]
    public class GlyphSettings
    {
        private const string ABOUT = "Demonstrates how to use the new CheckBoxImageGlyphInfo and RadioButtonImageGlyphInfo classes to render custom images for the various states exhibited by a CheckBox or RadioButton.";
        private static readonly Size defaultSize = new Size(13, 13);

        private GlyphType _glyphType = GlyphType.CheckBox;
        private Size _size = Size.Empty;
        private Color _baseColor = Color.Empty;
        private GlyphAppearances _checkedAppearances = null;
        private GlyphAppearances _uncheckedAppearances = null;

        private Image[] _checkedImages = null;
        private Image[] _uncheckedImages = null;
        private Image[] _indeterminateImages = null;

        public GlyphSettings(GlyphType glyphType)
        {
            this._glyphType = glyphType;
        }

        [Description(GlyphSettings.ABOUT)]
        [Category("Misc")]
        public string About
        {
            get { return string.Empty; }
        }

        [Description("Gets/sets the color on which the background, foreground, and border colors are derived when no specific setting exists. Use a dark color for best results.")]
        [Category("Appearance")]
        public Color BaseColor
        {
            get
            {
                //  Use non-standard colors to make it immediately apparent
                //  that any visual aspect of the CheckBox or RadioButton can
                //  be modified.
                return this._baseColor.IsEmpty ?
                    this._glyphType == GlyphType.CheckBox ?
                    Color.RoyalBlue :
                    Color.SeaGreen :
                    this._baseColor;
            }
            set
            {
                if (value != this._baseColor)
                {
                    this._baseColor = value;
                    this.OnPropChange(this, EventArgs.Empty);
                }
            }
        }

        [Description("Gets/sets the size at which the glyph will be displayed. Use odd-numbered dimensions for best results, so the value marker is centered properly.")]
        [Category("Metrics")]
        public Size Size
        {
            get { return this._size.IsEmpty == false ? this._size : defaultSize; }
            set
            {
                if (value != this._size)
                {
                    this._size = value;
                    this.OnPropChange(this, EventArgs.Empty);
                }
            }
        }

        [Description("Returns a GlyphAppearances object specific to the 'Checked' state.")]
        [Category("Appearance")]
        public GlyphAppearances CheckedAppearances
        {
            get
            {
                if (this._checkedAppearances == null)
                    this._checkedAppearances = new GlyphAppearances(CheckState.Checked, new EventHandler(this.OnPropChange));

                return this._checkedAppearances;
            }
        }

        [Description("Returns a GlyphAppearances object specific to the 'Unchecked' state.")]
        [Category("Appearance")]
        public GlyphAppearances UncheckedAppearances
        {
            get
            {
                if (this._uncheckedAppearances == null)
                    this._uncheckedAppearances = new GlyphAppearances(CheckState.Unchecked, new EventHandler(this.OnPropChange));

                return this._uncheckedAppearances;
            }
        }

        protected virtual GlyphAppearances GetAppearances(CheckState checkState)
        {
            return checkState == CheckState.Checked ? this.CheckedAppearances : this.UncheckedAppearances;
        }

        protected Image GetImage(CheckState checkState, VisualState visualState)
        {
            int index = (int)visualState;
            Image[] images = null;
            int supportedStates = Enum.GetValues(typeof(VisualState)).Length;

            switch (checkState)
            {
                case CheckState.Checked:
                    {
                        if (this._checkedImages == null)
                            this._checkedImages = new Image[supportedStates];

                        images = this._checkedImages;
                    }
                    break;

                case CheckState.Unchecked:
                    {
                        if (this._uncheckedImages == null)
                            this._uncheckedImages = new Image[supportedStates];

                        images = this._uncheckedImages;
                    }
                    break;

                case CheckState.Indeterminate:
                    {
                        if (this._indeterminateImages == null)
                            this._indeterminateImages = new Image[supportedStates];

                        images = this._indeterminateImages;
                    }
                    break;
            }

            if (images[index] == null)
            {
                GlyphAppearances glyphAppearances = this.GetAppearances(checkState);

                GlyphAppearance appearance = glyphAppearances.GetAppearance(visualState);

                Color baseColor = this.BaseColor;
                Color backColor = GetBackColor(glyphAppearances, visualState);
                Color foreColor = GetForeColor(glyphAppearances, visualState);
                Color borderColor = GetBorderColor(glyphAppearances, visualState);

                if (foreColor.IsEmpty)
                {
                    foreColor = checkState == CheckState.Indeterminate ?
                        Color.FromArgb(128, baseColor) :
                        baseColor;
                }

                if (backColor.IsEmpty)
                {
                    //backColor = Color.FromArgb(16, foreColor);
                    backColor = Color.White;
                }

                if (borderColor.IsEmpty)
                {
                    borderColor = Color.FromArgb(192, foreColor);
                }

                Size size = this.Size;
                Bitmap bitmap = new Bitmap(size.Width, size.Height);
                Rectangle bounds = new Rectangle(Point.Empty, size);
                bounds.Width -= 1;
                bounds.Height -= 1;

                bool isCheckBox = this._glyphType == GlyphType.CheckBox;

                using (Graphics gr = Graphics.FromImage(bitmap))
                {
                    gr.SmoothingMode = SmoothingMode.AntiAlias;

                    using (SolidBrush brush = new SolidBrush(backColor))
                    {
                        if (isCheckBox)
                            gr.FillRectangle(brush, bounds);
                        else
                            gr.FillEllipse(brush, bounds);
                    }

                    using (Pen pen = new Pen(borderColor))
                    {
                        if (isCheckBox)
                            gr.DrawRectangle(pen, bounds);
                        else
                            gr.DrawEllipse(pen, bounds);
                    }

                    //  Only draw the foreground color if the state is not 'Unchecked'.
                    if (checkState != CheckState.Unchecked)
                    {
                        using (SolidBrush brush = new SolidBrush(foreColor))
                        {
                            Rectangle rect = bounds;
                            int deflation = isCheckBox ? -2 : -3;
                            rect.Inflate(deflation, deflation);

                            if (isCheckBox)
                            {
                                if (checkState == CheckState.Indeterminate)
                                    gr.FillRectangle(brush, rect);
                                else
                                    if (checkState == CheckState.Checked)
                                    {
                                        using (Pen pen = new Pen(brush, 2f))
                                        {
                                            Point point1 = new Point(rect.X, rect.Y + (rect.Height / 2));
                                            Point point2 = new Point(rect.X + (rect.Width / 2), rect.Bottom);
                                            gr.DrawLine(pen, point1, point2);

                                            point1 = point2;
                                            point2 = new Point(rect.Right, rect.Y);
                                            gr.DrawLine(pen, point1, point2);
                                        }
                                    }
                            }
                            else
                                gr.FillEllipse(brush, rect);
                        }
                    }
                }

                images[index] = bitmap;
            }

            return images[index];

        }

        public override string ToString()
        {
            return string.Empty;
        }

        protected void DisposeImages()
        {
            int supportedStates = Enum.GetValues(typeof(VisualState)).Length;
            for (int i = 0; i < supportedStates; i++)
            {
                if (this._checkedImages != null && this._checkedImages[i] != null)
                {
                    this._checkedImages[i].Dispose();
                    this._checkedImages[i] = null;
                }

                if (this._uncheckedImages != null && this._uncheckedImages[i] != null)
                {
                    this._uncheckedImages[i].Dispose();
                    this._uncheckedImages[i] = null;
                }

                if (this._indeterminateImages != null && this._indeterminateImages[i] != null)
                {
                    this._indeterminateImages[i].Dispose();
                    this._indeterminateImages[i] = null;
                }
            }
        }

        static private Color GetBackColor(GlyphAppearances glyphInfo, VisualState visualState)
        {
            GlyphAppearance appearance = glyphInfo.GetAppearance(visualState);
            Color backColor = appearance.BackColor;

            if (backColor.IsEmpty)
            {
                VisualState nextState = GetFallbackState(visualState);
                if (nextState != visualState)
                {
                    appearance = glyphInfo.GetAppearance(nextState);
                    return GetBackColor(glyphInfo, nextState);
                }
            }
            else
                return backColor;

            return Color.Empty;
        }

        static private Color GetForeColor(GlyphAppearances glyphInfo, VisualState visualState)
        {
            GlyphAppearance appearance = glyphInfo.GetAppearance(visualState);
            Color foreColor = appearance.ForeColor;

            if (foreColor.IsEmpty)
            {
                VisualState nextState = GetFallbackState(visualState);
                if (nextState != visualState)
                {
                    appearance = glyphInfo.GetAppearance(nextState);
                    return GetForeColor(glyphInfo, nextState);
                }
            }
            else
                return foreColor;

            return Color.Empty;
        }

        static private Color GetBorderColor(GlyphAppearances glyphAppearances, VisualState visualState)
        {
            GlyphAppearance appearance = glyphAppearances.GetAppearance(visualState);
            Color borderColor = appearance.BorderColor;

            if (borderColor.IsEmpty)
            {
                VisualState nextState = GetFallbackState(visualState);
                if (nextState != visualState)
                {
                    appearance = glyphAppearances.GetAppearance(nextState);
                    return GetBorderColor(glyphAppearances, nextState);
                }
            }
            else
                return borderColor;

            return Color.Empty;
        }

        static private VisualState GetFallbackState(VisualState visualState)
        {
            switch (visualState)
            {
                case VisualState.PressedAndOver:
                    return VisualState.Pressed;

                default:
                    return VisualState.Normal;
            }
        }

        protected virtual void OnPropChange(object sender, EventArgs e) { }

        public virtual GlyphInfoBase ToGlyphInfo()
        {
            return null;
        }
    }
    #endregion GlyphSettings class

    #region VisualState enumeration
    /// <summary>
    /// Enumerates the various visual states for a CheckBox or RadioButton.
    /// </summary>
    public enum VisualState
    {
        Normal,
        Disabled,
        Pressed,
        MouseOver,
        PressedAndOver,
    }
    #endregion VisualState enumeration
}
